//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.negotiation.commands.*;

import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import java.util.*;
import java.math.*;
import java.sql.*;
 
import com.ibm.commerce.command.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.negotiation.util.*;
import com.ibm.commerce.negotiation.objects.*;
import com.ibm.commerce.negotiation.operation.*;
import com.ibm.commerce.negotiation.beans.*;
import com.ibm.commerce.negotiation.objimpl.*;
import com.ibm.commerce.negotiation.misc.*;
 
/**
 *
 * Determines the winning auto-bids's next bidding price for an auction.
 * @author: Kevin Zhang 
 */ 
public class DetermineWinnersCmdImpl extends ECCommandImpl
								     implements DetermineWinnersCmd, AuctionConstants
{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	private Vector _activeBids = null;
	private Double _quantityOnAuction = null;
	private Long   _bidRuleId = null;
	private String _errorTaskName = null;

	private ControlRuleAccessBean _bidRule = null;
	//a list of auto bids that are winning. set by the caller command which is AutoBidTaskCmdImpl
	protected java.util.Vector winners;
/**
 * DetermineWinnersCmdImpl default constructor.
 */
public DetermineWinnersCmdImpl()
{
	super();
}
/**
 * Calculates the next bid value based on the current maximum bid value
 * and the bid rule range/increment.
 * @param BigDecimal currMaxBidValue - Current maximum bid value.
 * @return BigDecimal - Calculated next bid value
 * @exception ECException - Invalid input bid value or, error during bid control rule bean lookup. 
 */
protected BigDecimal calculateNextBidValue(BigDecimal currMaxBidValue) throws ECException
{
	final String methodName = "calculateNextBidValue";

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName );

	if ( (currMaxBidValue == null) || (currMaxBidValue.signum() < 0) )
	{
		TypedProperty exceptFields = new TypedProperty();
		String key = AuctionServerMessageHelper.getMessage(this.getCommandContext().getLocale(),BID_VAL_KEY );
		exceptFields.put(key, currMaxBidValue );			
		throw new ECApplicationException( ECMessage._ERR_INVALID_BID_VALUE,				
		                                  this.getClass().getName(), methodName,
		                                  ECMessageHelper.generateMsgParms(currMaxBidValue),
		                                  getErrorTaskName(),
										  exceptFields);
	}

	// Default calculated bid value when all attempts fail.
	BigDecimal calcBidVal = currMaxBidValue.add( getDefaultBidIncrement() );			
	
	ControlRuleAccessBean controlRuleAccessBean = null;
	BigDecimal minBidVal = getDefaultMinBidValue();
	
	if (getBidRuleId() != null)
	{
		try
		{
			// Obtain Control Rule bean
			controlRuleAccessBean = getBidRule();

			minBidVal = controlRuleAccessBean.getMinimumValueInEJBType();

			// Compute bid value based on maximum bid value and value range/increment.
			if ( (controlRuleAccessBean != null) &&
			     (controlRuleAccessBean.getRuleType().equals(EC_AUCTION_OPEN_CRY_TYPE)))
			{
				RuleInterpreter interpreter = RuleInterpreterFactory.createRuleInterpreter(controlRuleAccessBean);
				interpreter.populateDataBean();
				ControlRuleDataBean controlRuleDataBean = interpreter.getControlRuleDataBean();

				NumericRangeDataBean bidValueRange[] = ((OpenCryBidControlRuleDataBean) controlRuleDataBean).getPriceRanges();

				int numOfRanges = bidValueRange.length;
				for (int i=0; i < numOfRanges; i++)
				{
					BigDecimal upperLimit = bidValueRange[i].getUpperLimit();
					BigDecimal lowerLimit = bidValueRange[i].getLowerLimit();
					BigDecimal increment = bidValueRange[i].getIncrement();;
					if ( ( currMaxBidValue.compareTo(lowerLimit) == 1) &&
						 ((currMaxBidValue.compareTo(upperLimit) == 0)	||
					     (currMaxBidValue.compareTo(upperLimit) == -1)))
					{
						calcBidVal = increment.add( currMaxBidValue );
						break;
					}
				}
			}
		}
		catch(FinderException fe)
		{
			// Control rule was not found.
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName, "Control Rule not found.");
		}
		catch(Exception e)
		{
			handleException( e );
		}
	}

	if (calcBidVal.compareTo(minBidVal) == -1)
	{
		// The minimum value is the lowest value that should be returned.
		calcBidVal = minBidVal;
	}
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName );

	return calcBidVal;
		
}
/**
 * Given the index of a winning auto bid, to find the closest loser(the runner-up) which is defined as 
 * the bid that is going to win if this winning autobid doesn't exist.
 * @return com.ibm.commerce.negotiation.objimpl.BidDataLight
 * @param start int
 * @param availQuant double
 */
protected BidDataLight findNextLoser(int start, double availQuant) 
{
	final String methodName = "findNextLoser";
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName);
	BidDataLight ret = null;
	Vector list = this.getActiveBids();
	//  The active bids list is already sorted. is set by the calling command AutoBidTaskCmd
	int sz = list.size();
	if(start >= sz)
		return null;
	for(int i = start+1; i < sz; i++)
	{
		BidDataLight bid = (BidDataLight)list.elementAt(i);
		
		if(bid.getWinningQuantity().equals(bid.getQuantity()))
		{
			//this is a winner, try next one.
			//Only if the bidder is getting the full quanlity, he/she is a true winner.
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName,
		          "found winner["+bid.getBidId()+"] Winning quantity: "+bid.getWinningQuantity());	
			continue;
		}
		if(bid.getQuantity().doubleValue() > availQuant && !bid.acceptPartialQuantity())
		{
			//this one has no chance to win since not enough quantity. try next one.
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName,
		          "found winner["+bid.getBidId()+"] Winning quantity: "+bid.getWinningQuantity());	
			continue;
		}
		else
		{
			//this one is the runner-up to win. return this one.
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName,
		          "found loser["+bid.getBidId()+"] Winning quantity: "+bid.getWinningQuantity());	
			ret = bid;
			break;
		}

	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName );

	return ret;
}
/**
 * Get the list of active bids for processing.
 * @return java.util.Vector
 */
protected Vector getActiveBids()
{
	return _activeBids;
}
/**
 * A singleton method for retrieving Control Rule Access Bean.
 * @return ControlRuleAccessBean - Control rule access bean, or null if
 *                                 current bid rule id is invalid.
 * @exception Exception - FinderException or data base retrieval exception.
 */
protected ControlRuleAccessBean getBidRule() throws Exception
{
	if ( (_bidRule == null) && (getBidRuleId() != null))
	{
		// Obtain Control Rule bean
		_bidRule = new ControlRuleAccessBean();
		_bidRule.setInitKey_id( getBidRuleId() );
		_bidRule.refreshCopyHelper();
	}
	
	return _bidRule;
}
/**
 * Gets the auction's Bid Rule Id.
 * @param Long - Bid rule Id.
 */
protected Long getBidRuleId()
{
	return(_bidRuleId);	
}
/**
 * Return a default bid increment value. This method should be overrided if the
 * the returned default value is not acceptable.
 * @return java.math.BigDecimal
 */
protected BigDecimal getDefaultBidIncrement()
{
	return new BigDecimal("1.0");
}
/**
 * Return a default minimum bid value. This method should be overrided if the
 * the returned default value is not acceptable.
 * @return java.math.BigDecimal
 */ 
protected BigDecimal getDefaultMinBidValue()
{
	return new BigDecimal("0.0");
}
/**
 * returns error task name that will be returned by the exceptions.
 * @return java.lang.String - Error task name
 */
protected String getErrorTaskName()
{
	return(_errorTaskName);
}
/**
 * Get the quantity available for auction.
 * @return Double - Quantity on auction.
 */
protected Double getQuantityOnAuction()
{
	return (_quantityOnAuction);
}
/**
 * Get the list of auto bids that are winning.
 * @return java.util.Vector
 */ 
public java.util.Vector getWinners() 
{
	return winners;
}
/**
 * Formats an ECException based on the input exception value.
 *
 * @param e Exception - The exception instance to be formatted.
 * @exception ECException 
 */
protected void handleException(Exception e) throws ECException
{
	final String methodName = "handleException";
	
	if (e != null)
	{
		if (e instanceof ECApplicationException)
		{
			throw (ECApplicationException)e;
		}
		else if (e instanceof ECSystemException)
		{
			throw (ECSystemException)e;
		}
		else if (e instanceof NullPointerException)
		{
			throw new ECSystemException(ECMessage._ERR_NULL_EXCEPTION,
				                        this.getClass().getName(), methodName,
				                        ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}		
		else if (e instanceof RemoteException)
		{
			throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION,
				                        this.getClass().getName(), methodName,
				                        ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		else if (e instanceof CreateException)
		{
			throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION,
				                        this.getClass().getName(),methodName,
				                        ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		else if (e instanceof FinderException)
		{
			throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION,
				                        this.getClass().getName(),methodName,
				                        ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		else if (e instanceof NamingException)
		{
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION,
				                        this.getClass().getName(),methodName,
				                        ECMessageHelper.generateMsgParms(e.getMessage()), e);
 	    }
		else
		{
			TypedProperty exceptFields = new TypedProperty();
			exceptFields.put(AuctionServerMessageHelper.getMessage(this.getCommandContext().getLocale(),COMMAND_NAME_KEY), this.getClass().getName());			
			throw new ECApplicationException(ECMessage._ERR_GENERIC,
				                             this.getClass().getName(),methodName,
				                             ECMessageHelper.generateMsgParms(e.getMessage()),
				                             getErrorTaskName(),
				                             exceptFields
				                            );
  	    }
	}
					
}
/**
 * Verifies that the this command is ready to be executed.
 * @return boolean =true if command is ready, otherwise false.
 */
public boolean isReadyToCallExecute()
{
	final String methodName = "isReadyToCallExecute";
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName);
	if (!super.isReadyToCallExecute())
		return false;

	if (_activeBids== null)
	{
		String key = AuctionServerMessageHelper.getMessage(this.getCommandContext().getLocale(), BID_DATA_KEY );
		ECMessageLog.out(ECMessage._ERR_DIDNT_SET_BID_PARM, this.getClass().getName(), methodName, key);
		return false;
	}
	
	if (_quantityOnAuction == null)
	{
		String key = AuctionServerMessageHelper.getMessage(this.getCommandContext().getLocale(), AUCTION_QUANTITY_KEY );
		ECMessageLog.out(ECMessage._ERR_DIDNT_SET_AUCTION_PARM, this.getClass().getName(), methodName, key);
		return false;
	}	
	
	if (_errorTaskName == null)
	{
		String key = AuctionServerMessageHelper.getMessage(this.getCommandContext().getLocale(), ERROR_TASK_NAME_KEY );
		ECMessageLog.out(ECMessage._ERR_DIDNT_SET_ERROR_TASK_NAME, this.getClass().getName(), methodName, key);
		return false;
	}			

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName);
	return true;
}
/**
 * For all active auto bids, determine if a new bid value should be submitted on behalf of the
 * auto bid, in order to put the auto bid in a winning position. 
 */
public void performExecute() throws ECException
{
	final String methodName = "performExecute";
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName);

	int index = 0;

	boolean allAutoBids = true;
	boolean allWinners = true;
		
	double remainingQuant = getQuantityOnAuction().doubleValue();
	int sz = getWinners().size();
//	BigDecimal minBid = null;
	for(int i = 0; i < sz; i++)
	{
		BidDataLight winner = (BidDataLight)getWinners().elementAt(i);
		BidDataLight loser = findNextLoser(winner.getIndex(), remainingQuant);
		BigDecimal priceToWin = null;
		if(loser == null)
		{
			//no competiter, can win with initial price
			priceToWin = winner.getInitBidPrice();
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName,
		          "no match with init bid price["+priceToWin+"] Winning quantity: "+winner.getWinningQuantity());	
		}
		else
		{
			//found loser.
			int comp = winner.getWinningQuantity().compareTo(loser.getQuantity());
			//In theory:
			//if the winner's bidding less quantity than loser, winner has to beat the loser's price.
			//if the winner's bidding same quatity as loser, and bid time is later than loser, winner has to beat the loser's price.
//			if(comp <0 || (comp==0 && winner.getBidTime().after(loser.getBidTime()))) 
			if(comp <=0 ) //since finalize auction won't look at autobid time, for now if quantity is same, winner has to beat loser on price.
			{
				BigDecimal nextBidPrice = calculateNextBidValue(loser.getMaxBidValue());
				if(nextBidPrice.compareTo(winner.getMaxBidValue()) > 0)
				{
					//if the bidder is bidding at max limit, he/she doesn't have to following the bid rule.
					priceToWin = winner.getMaxBidValue();
				}
				else
				{
					priceToWin = nextBidPrice;
				}
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName,
		          winner.getBidId()+" beating"+loser.getBidId()+" with priceToWin["+priceToWin+"] Winning quantity: "+winner.getWinningQuantity());	
			}
			else
			{
				//only need to match the price.
				priceToWin = loser.getMaxBidValue();
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName,
			          winner.getBidId()+"matching "+loser.getBidId()+" with priceToWin["+priceToWin+"] Winning quantity: "+winner.getWinningQuantity());	
			}//
		}//end of found loser.
		
		if(priceToWin.compareTo(winner.getInitBidPrice()) < 0)
		{
			//bidding price can not below initial bid price.
			priceToWin = winner.getInitBidPrice();
		}
		winner.setPriceToWin(priceToWin);
		remainingQuant -= winner.getWinningQuantity().doubleValue();
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName );
}
/**
 * Resets this command to allow for future reuse.
 */
public void reset()
{
	super.reset();

	_bidRule           = null;		
	_quantityOnAuction = null;
	_bidRuleId         = null;
	_errorTaskName     = null;
}
/**
 * Sets the list of active bids to be processed.
 * @param bids Vector - Active bids list
 */
public void setActiveBids(Vector bids)
{
	_activeBids = bids;
}
/**
 * Sets the auction Bid Rule id
 * @param id Long - Bid rule id
 */
public void setBidRuleId(Long id)
{
	_bidRuleId = id;	
}
/**
 * Set error task name that will be returned by the exceptions.
 * @param errorTaskName java.lang.String
 */
public void setErrorTaskName(String errorTaskName)
{
	_errorTaskName = errorTaskName;
}
/**
 * Set the quantity available for auction.
 * @param quantity Double - Quantity available for auction.
 */
public void setQuantityOnAuction(Double quantity)
{
	_quantityOnAuction = quantity;
}
/**
 * Set the list of autobids that are currently winning.
 * @param newWinners java.util.Vector
 */
public void setWinners(java.util.Vector newWinners) 
{
	winners = newWinners;
}
/**
 * Return a string that shows the information about a bid.
 * @param bid BidDataLight
 * @return String
 */
public static String traceBid(BidDataLight bid) 
{
	if(bid == null)
		return "null";
	StringBuffer str = new StringBuffer("BidId(");
	str.append(bid.getBidId());
	str.append(")AutoBidId(");
	str.append(bid.getAuctionId());
	str.append(")MaxBidVal(");
	str.append(bid.getMaxBidValue());
	str.append(")CurrBidVal(");
	str.append(bid.getCurrentBidValue());
	str.append(")BidQuant(");
	str.append(bid.getQuantity());
	str.append(")TakePartial(");
	str.append(bid.acceptPartialQuantity());
	str.append(")WinQuant(");
	str.append(bid.getWinningQuantity());
	str.append(")PriceToWin(");
	str.append(bid.getPriceToWin());
	str.append(")");

	return str.toString();
}
}
