//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.tools.campaigns.*;

import com.ibm.commerce.accesscontrol.AccessVector;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.condition.AndListCondition;
import com.ibm.commerce.condition.Condition;
import com.ibm.commerce.condition.ConditionConstants;
import com.ibm.commerce.condition.OpenCondition;
import com.ibm.commerce.condition.OrListCondition;
import com.ibm.commerce.condition.SimpleCondition;
import com.ibm.commerce.condition.TrueCondition;
import com.ibm.commerce.copyright.IBMCopyright;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ParameterNotFoundException;
import com.ibm.commerce.marketingcenter.objects.InitiativeAccessBean;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECMessageHelper;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;
import com.ibm.commerce.rule.Action;
import com.ibm.commerce.rule.Rule;
import com.ibm.commerce.server.ECConstants;
import com.ibm.commerce.tools.command.ToolsControllerCommandImpl;
import com.ibm.commerce.tools.common.ECToolsConstants;
import com.ibm.commerce.tools.common.ui.UIProperties;
import com.ibm.commerce.tools.util.ResourceDirectory;
import com.ibm.commerce.utils.TimestampHelper;
import java.sql.Timestamp;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * The CampaignInitiativeSaveControllerCmdImpl class implements the
 * CampaignInitiativeSaveControllerCmd interface to
 * provide the controller command that saves a campaign initiative.
 * Different implementations of this interface
 * can be provided for different stores.
 * This controller command is mapped to the CampaignInitiativeSave url.
 */
public class CampaignInitiativeSaveControllerCmdImpl extends ToolsControllerCommandImpl implements CampaignInitiativeSaveControllerCmd, ECConstants, ECToolsConstants, CampaignConstants, ConditionConstants {
  /**
   * Copyright.
   */
  public static final String COPYRIGHT = IBMCopyright.SHORT_COPYRIGHT;

  /**
   * An instance of CampaignInitiativeDetailsDataBean.
   */
  protected CampaignInitiativeDetailsDataBean _initiative = null;

  /**
   * XML document that describes the campaign initiative.
   */
  protected String _xmlFile = null;

  /**
   * This flag indicates that the controller command should try to force the save.
   */
  protected boolean _forceSave = false;

  /**
   * This flag indicates that the controller command should do a create or an update.
   */
  protected boolean _newInitiative = false;

  /**
   * A vector of campaign initiative segments.
   */
  protected Vector _initiativeSegments = new Vector();

  /**
   * XML document that describes the initiative rule.
   */
  protected String _initiativeRule = null;

  /**
   * An instance of SimpleCondition.Qualifier which is generated using the given languages.
   */
  protected SimpleCondition.Qualifier _languageQualifier = null;

  /**
   * An instance of SimpleCondition.Qualifier which is generated using the supplied currencies.
   */
  protected SimpleCondition.Qualifier _currencyQualifier = null;

  /**
   * Contains the error condition detected by this controller command.
   */
  protected String _error = null;

  /**
   * Campaign initiative ID.
   */
  protected Integer _initiativeId = null;

  /**
   * The view task name from the response.
   */
  protected String _responseViewTaskName = null;

  /**
   * Campaign initiative result type.
   */
  protected String _resulttype = null;

  /**
   * Parses and validates the parameters passed to this controller command.
   * The following parameters are valid:
   * <ul>
   * <li><code>xml</code> - An XML document that describes the campaign initiative.</li>
   * </ul>
   */
  public void validateParameters () throws ECException {
	final String methodName = "validateParameters";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
	try {
		Vector xmlObjectVector = (Vector) requestProperties.get(EC_XMLOBJECT);
		if (xmlObjectVector == null || xmlObjectVector.elementAt(0) == null) {
			throw new ParameterNotFoundException(EC_XMLOBJECT);
		}
		Hashtable xml = (Hashtable) (((Hashtable)xmlObjectVector.elementAt(0)).get(EC_XML));
		_xmlFile = (String) xml.get(EC_XMLFILE);
		_newInitiative = CampaignUtil.toBoolean((String) xml.get(PARAMETER_NEW_INITIATIVE));
		_forceSave = CampaignUtil.toBoolean((String) xml.get(ELEMENT_FORCE_SAVE));
		_initiative = new CampaignInitiativeDetailsDataBean();
		_initiative.setXML((Hashtable) xml.get(ELEMENT_INITIATIVE));
		try {
			_initiative.populate();
		}
		catch (Exception e) {
			throw new ParameterNotFoundException(ELEMENT_INITIATIVE);
		}
	}
	catch (ParameterNotFoundException e) {
		throw new ECApplicationException(ECMessage._ERR_CMD_MISSING_PARAM, getClass().getName(), methodName, ECMessageHelper.generateMsgParms(e.getParamName()));
	}

	_initiativeId = _initiative.getId();
	if (_initiativeId == null && _newInitiative == false) {
		_responseViewTaskName = "WizardNavigation";
	}
	else {
		_responseViewTaskName = "NotebookNavigation";
	}
	// check to see if the campaign initiative can be saved
	if (_newInitiative || _initiativeId == null) {
		_initiativeId = null;
		// check if the name already exists
		try {
			InitiativeAccessBean initiativeAccessBean = new InitiativeAccessBean().findByStoreIdAndName(getCommandContext().getStoreId(), _initiative.getInitiativeName());
			if (initiativeAccessBean != null) {
				if (_forceSave) {
					_initiativeId = initiativeAccessBean.getInitiativeIdInEJBType();
				}
				else {
					_error = ERROR_INITIATIVE_EXISTS;
				}
			}
		}
		catch (Exception e) {
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName, e.toString());
		}
	}
	else {
		// check if the name already exists and is not this initiative
		try {
			InitiativeAccessBean initiativeAccessBean = new InitiativeAccessBean().findByStoreIdAndName(getCommandContext().getStoreId(), _initiative.getInitiativeName());
			if (initiativeAccessBean != null) {
				if (_forceSave) {
					_initiativeId = initiativeAccessBean.getInitiativeIdInEJBType();
				}
				else if (_initiativeId.equals(initiativeAccessBean.getInitiativeIdInEJBType()) == false) {
					_error = ERROR_INITIATIVE_EXISTS;
				}
			}
		}
		catch (Exception e) {
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName, e.toString());
		}
		// check if someone else changed it
		if (_error == null) {
			try {
				InitiativeAccessBean initiativeAccessBean = new InitiativeAccessBean();
				initiativeAccessBean.setInitKey_initiativeId(_initiativeId.toString());
				Long lastUpdateDate = new Long(initiativeAccessBean.getLastUpdateDateInEJBType().getTime());
				if (!_forceSave && !lastUpdateDate.equals(_initiative.getLastUpdateDate())) {
					_error = ERROR_INITIATIVE_CHANGED;
				}
			}
			catch (Exception e) {
				_initiativeId = null;
			}
		}
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
  }

  /**
   * Gets the resources that are accessed by this controller command.
   * The campaign initiative access beans for the specified campaign initiative ids are
   * the resource returned by this method.
   *
   * @return A vector of resources.
   */
  public AccessVector getResources () throws ECException {
	final String methodName = "getResources";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
	AccessVector accessVector = null;
	if (_error == null) {
		if (_initiativeId != null) {
			InitiativeAccessBean initiativeAccessBean = new InitiativeAccessBean();
			initiativeAccessBean.setInitKey_initiativeId(_initiativeId.toString());
			accessVector = new AccessVector(initiativeAccessBean);
		}
		else {
			accessVector = new AccessVector(getCommandContext().getStore());
		}
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
	return accessVector;
  }

  /**
   * The performExecute method is called to when the marketing manager
   * asks to save a campaign initiative.
   */
  public void performExecute () throws ECException {
	final String methodName = "performExecute";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
	super.performExecute();
	String errorMessage = null;

	// go ahead and save the campaign initiative
	if (_error == null) {
		constructRule();
		String status = null;
		if (_initiative.getDisabled()) {
			status = INITIATIVE_STATUS_DISABLED;
		}
		try {
			if (_initiativeId == null) {
				CampaignInitiativeCreateCmd createCmd = (CampaignInitiativeCreateCmd) CommandFactory.createCommand(CampaignInitiativeCreateCmd.Name, getCommandContext().getStoreId());
				createCmd.setCommandContext(getCommandContext());
				createCmd.setInitiativeName(_initiative.getInitiativeName());
				createCmd.setInitiativeDescription(_initiative.getDescription());
				createCmd.setInitiativeCampaignId(_initiative.getCampaignId());
				createCmd.setInitiativeRule(_initiativeRule);
				createCmd.setInitiativeSegments(_initiativeSegments);
				createCmd.setInitiativeSellType(_initiative.getSellType());
				createCmd.setInitiativeStatus(status);
				createCmd.setInitiativeResultType(_resulttype);
				createCmd.execute();
			}
			else {
				CampaignInitiativeUpdateCmd updateCmd = (CampaignInitiativeUpdateCmd) CommandFactory.createCommand(CampaignInitiativeUpdateCmd.Name, getCommandContext().getStoreId());
				updateCmd.setCommandContext(getCommandContext());
				updateCmd.setInitiativeId(_initiativeId);
				updateCmd.setInitiativeName(_initiative.getInitiativeName());
				updateCmd.setInitiativeDescription(_initiative.getDescription());
				updateCmd.setInitiativeCampaignId(_initiative.getCampaignId());
				updateCmd.setInitiativeRule(_initiativeRule);
				updateCmd.setInitiativeSegments(_initiativeSegments);
				updateCmd.setInitiativeSellType(_initiative.getSellType());
				updateCmd.setInitiativeStatus(status);
				updateCmd.setInitiativeResultType(_resulttype);
				updateCmd.execute();
			}
		}
		catch (ECException e) {
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName, e.toString());
			_error = ERROR_INITIATIVE_NOT_SAVED;
			Hashtable resources = (Hashtable) ResourceDirectory.lookup(CAMPAIGNS_RESOURCES, getCommandContext().getLocale());
			errorMessage = (String) resources.get(MSG_INITIATIVE_NOT_SAVED);
		}
	}

	// set response property
	responseProperties = new TypedProperty();
	responseProperties.put(EC_VIEWTASKNAME, _responseViewTaskName);
	if (_error != null) {
		responseProperties.put(UIProperties.SUBMIT_ERROR_STATUS, _error);
	}
	else {
		// creation confirmation
		if (_responseViewTaskName.equals("WizardNavigation")) {
			Hashtable resources = (Hashtable) ResourceDirectory.lookup(CAMPAIGNS_RESOURCES, getCommandContext().getLocale());
			String finishMessage = (String) resources.get(MSG_INITIATIVE_WIZARD_SUCCESS_CONFIRMATION);
			responseProperties.put(UIProperties.SUBMIT_FINISH_MESSAGE, finishMessage);
		}
	}
	if (errorMessage != null) {
		responseProperties.put(UIProperties.SUBMIT_ERROR_MESSAGE, errorMessage);
	}
	responseProperties.put(EC_XMLFILE, _xmlFile);

	if (_error == ERROR_INITIATIVE_NOT_SAVED) {
		// rollback the transaction
		throw new ECApplicationException(ECCampaignsMessage.ERROR_INITIATIVE_NOT_SAVED, getClass().getName(), methodName, null, _responseViewTaskName, responseProperties);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MARKETING, getClass().getName(), methodName);
  }

  /**
   * The constructRule method is called to construct the XML document that contains
   * all the conditions with this initiative.
   */
  protected void constructRule () {
	final String methodName = "constructRule";
	// create the initiative rule

	Vector conditionVector = new Vector();

	// WHO
	boolean targetAllShoppers = _initiative.getTargetAllShoppers();
	Vector selectedSegments = _initiative.getSelectedSegments();
	if (!targetAllShoppers && selectedSegments.size() > 0) {
		Condition segmentConditions[] = new Condition[selectedSegments.size()];
		for (int i=0; i<selectedSegments.size(); i++) {
			String segment = (String) selectedSegments.elementAt(i);
			segmentConditions[i] = new SimpleCondition(VARIABLE_SEGMENT, OPERATOR_EQUAL_TO, segment);
			boolean found = false;
			for (int j=0; j<_initiativeSegments.size(); j++) {
				if (_initiativeSegments.elementAt(j).equals(segment)) {
					found = true;
					break;
				}
			}
			if (!found) {
				_initiativeSegments.addElement(segment);
			}
		}
		if (segmentConditions.length == 1) {
			conditionVector.addElement(segmentConditions[0]);
		}
		else {
			conditionVector.addElement(new OrListCondition(segmentConditions));
		}
	}

	// WHEN
	Vector whenChoices = _initiative.getWhenChoices();
	if (whenChoices.size() > 0) {
		for (int whenIndex=0; whenIndex<whenChoices.size(); whenIndex++) {
			CampaignInitiativeDetailsDataBean.WhenChoice whenChoice = (CampaignInitiativeDetailsDataBean.WhenChoice) whenChoices.elementAt(whenIndex);
			String action = whenChoice.getAction();
			String type = whenChoice.getType();
			String value1 = whenChoice.getValue1();
			String value2 = whenChoice.getValue2();

			if (action.equals(SHOPPING_CART_CONTAINS)) {
				// shopping cart contains
				if (type.equals(SKU)) {
					conditionVector.addElement(new SimpleCondition(VARIABLE_SHOPPING_CART_SKU, OPERATOR_EQUAL_TO, value1));
				}
				else if (type.equals(CATEGORY)) {
					conditionVector.addElement(new SimpleCondition(VARIABLE_SHOPPING_CART_CATEGORY, OPERATOR_EQUAL_TO, value1, getLanguageQualifier()));
				}
			}
			else if (action.equals(SHOPPING_CART_DOESNOT_CONTAIN)) {
				// shopping cart does not contain
				if (type.equals(SKU)) {
					conditionVector.addElement(new SimpleCondition(VARIABLE_SHOPPING_CART_SKU, OPERATOR_NOT_EQUAL_TO, value1));
				}
				else if (type.equals(CATEGORY)) {
					conditionVector.addElement(new SimpleCondition(VARIABLE_SHOPPING_CART_CATEGORY, OPERATOR_NOT_EQUAL_TO, value1, getLanguageQualifier()));
				}
			}
			else if (action.equals(PURCHASE_HISTORY_CONTAINS)) {
				// purchase history contains
				if (type.equals(SKU)) {
					conditionVector.addElement(new SimpleCondition(VARIABLE_PURCHASE_HISTORY_SKU, OPERATOR_EQUAL_TO, value1));
				}
				else if (type.equals(CATEGORY)) {
					OpenCondition.Parameter[] parameters = new OpenCondition.Parameter[3];
					parameters[0] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_COMPARISON_TYPE, OPERATOR_EQUAL_TO);
					parameters[1] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_VALUE, value1);
					parameters[2] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_LANGUAGE, getCommandContext().getLocale().toString());
					conditionVector.addElement(new OpenCondition(OPEN_CONDITION_PURCHASE_HISTORY_CATEGORY, parameters));
				}
			}
			else if (action.equals(PURCHASE_HISTORY_DOESNOT_CONTAIN)) {
				// purchase history does not contain
				if (type.equals(SKU)) {
					conditionVector.addElement(new SimpleCondition(VARIABLE_PURCHASE_HISTORY_SKU, OPERATOR_NOT_EQUAL_TO, value1));
				}
				else if (type.equals(CATEGORY)) {
					OpenCondition.Parameter[] parameters = new OpenCondition.Parameter[3];
					parameters[0] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_COMPARISON_TYPE, OPERATOR_NOT_EQUAL_TO);
					parameters[1] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_VALUE, value1);
					parameters[2] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_LANGUAGE, getCommandContext().getLocale().toString());
					conditionVector.addElement(new OpenCondition(OPEN_CONDITION_PURCHASE_HISTORY_CATEGORY, parameters));
				}
			}
			else if (action.equals(SHOPCART_TOTAL_LESSTHAN)) {
				OpenCondition.Parameter[] parameters = new OpenCondition.Parameter[3];
				parameters[0] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_COMPARISON_TYPE, OPERATOR_LESS_THAN);
				parameters[1] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_VALUE, value1);
				parameters[2] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_CURRENCY, _initiative.getCurrency());
				conditionVector.addElement(new OpenCondition(OPEN_CONDITION_SHOPPING_CART_TOTAL, parameters));
			}
			else if (action.equals(SHOPCART_TOTAL_GREATERTHAN)) {
				OpenCondition.Parameter[] parameters = new OpenCondition.Parameter[3];
				parameters[0] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_COMPARISON_TYPE, OPERATOR_GREATER_THAN);
				parameters[1] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_VALUE, value1);
				parameters[2] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_CURRENCY, _initiative.getCurrency());
				conditionVector.addElement(new OpenCondition(OPEN_CONDITION_SHOPPING_CART_TOTAL, parameters));
			}
			else if (action.equals(SHOPCART_TOTAL_EQUALTO)) {
				OpenCondition.Parameter[] parameters = new OpenCondition.Parameter[3];
				parameters[0] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_COMPARISON_TYPE, OPERATOR_EQUAL_TO);
				parameters[1] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_VALUE, value1);
				parameters[2] = new OpenCondition.Parameter(OPEN_CONDITION_PARAMETER_CURRENCY, _initiative.getCurrency());
				conditionVector.addElement(new OpenCondition(OPEN_CONDITION_SHOPPING_CART_TOTAL, parameters));
			}
		}
	}

	// WHEN - day
	Vector daysOfTheWeek = _initiative.getDaysOfTheWeek();
	if (!_initiative.getEveryday() && daysOfTheWeek != null && daysOfTheWeek.size() > 0) {
		if (daysOfTheWeek.size() > 1) {
			Condition [] dayConditions = new Condition[daysOfTheWeek.size()];
			for (int i=0; i<daysOfTheWeek.size(); i++) {
				dayConditions[i] = new SimpleCondition(VARIABLE_DAY_OF_WEEK, OPERATOR_EQUAL_TO, (String) daysOfTheWeek.elementAt(i));
			}
			conditionVector.addElement(new OrListCondition(dayConditions));
		}
		else {
			conditionVector.addElement(new SimpleCondition(VARIABLE_DAY_OF_WEEK, OPERATOR_EQUAL_TO, (String) daysOfTheWeek.elementAt(0)));
		}
	}

	Condition condition = null;
	if (conditionVector.size() == 0) {
		condition = new TrueCondition();
	}
	else if (conditionVector.size() == 1) {
		condition = (Condition) conditionVector.elementAt(0);
	}
	else {
		Condition[] conditionList = new Condition[conditionVector.size()];
		conditionVector.copyInto(conditionList);
		condition = new AndListCondition(conditionList);
	}

	String whatType = _initiative.getWhatType();
	String actionName = null;
	Vector parameterVector = new Vector();

	// WHAT
	if (WHAT_TYPE_SPECIFIC_PRODUCTS.equals(whatType)) {
		_resulttype = INITIATIVE_RESULT_TYPE_PRODUCT;
		actionName = ACTION_SUGGESTIVE_SELL;
		Vector productList = _initiative.getProductSkus();
		for (int prodIndex=0; prodIndex<productList.size(); prodIndex++) {
			parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_SKU, (String) productList.elementAt(prodIndex)));
		}
		parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_QUERY_OPERATOR, ACTION_PARAMETER_VALUE_OR));
		parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_QUERY_TYPE, ACTION_PARAMETER_VALUE_SKU_QUERY));
	}
	else if (WHAT_TYPE_PRODUCT_ATTRIBUTES.equals(whatType)) {
		_resulttype = INITIATIVE_RESULT_TYPE_PRODUCT;
		actionName = ACTION_SUGGESTIVE_SELL;
		// generic query
		Vector productAttributeList = _initiative.getProductAttributes();

		for (int attrIndex=0; attrIndex<productAttributeList.size(); attrIndex++) {
			CampaignInitiativeDetailsDataBean.WhenChoice whenChoice = (CampaignInitiativeDetailsDataBean.WhenChoice) productAttributeList.elementAt(attrIndex);
			String type = whenChoice.getType();
			String value1 = whenChoice.getValue1();
			String value2 = whenChoice.getValue2();

			if (value1.length() > 0) {
				if (type.equals(CATEGORY)) {
					Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_CATEGORY, value1);
					Action.Parameter subParameters[] = new Action.Parameter[1];
					subParameters[0] = new Action.Parameter(ACTION_PARAMETER_LANGUAGE, CampaignUtil.getLocaleNameFromLanguageId(value2));
					parameter.setSubParameters(subParameters);
					parameterVector.addElement(parameter);
				}
				else if (type.equals(PRODUCT_DESCRIPTION)) {
					// product description
					Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_PRODUCT_DESCRIPTION, value1);
					Action.Parameter subParameters[] = new Action.Parameter[1];
					subParameters[0] = new Action.Parameter(ACTION_PARAMETER_LANGUAGE, CampaignUtil.getLocaleNameFromLanguageId(value2));
					parameter.setSubParameters(subParameters);
					parameterVector.addElement(parameter);
				}
				else if (type.equals(SKU)) {
					// sku begins with
					Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_SKU_PREFIX, value1);
					parameterVector.addElement(parameter);
				}
				else if (type.equals(LOW_INVENTORY)) {
					// inventory
					String comparisonOperator = null;
					if (value1.equals(GREATER_THAN)) {
						comparisonOperator = OPERATOR_GREATER_THAN;
					}
					else if (value1.equals(EQUAL_TO)) {
						comparisonOperator = OPERATOR_EQUAL_TO;
					}
					else if (value1.equals(LESS_THAN)) {
						comparisonOperator = OPERATOR_LESS_THAN;
					}
					if (comparisonOperator !=  null) {
						Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_INVENTORY_QUANTITY, value2);
						Action.Parameter subParameters[] = new Action.Parameter[1];
						subParameters[0] = new Action.Parameter(ACTION_PARAMETER_COMPARISON_OPERATOR, comparisonOperator);
						parameter.setSubParameters(subParameters);
						parameterVector.addElement(parameter);
					}
				}
				else if (type.equals(HIGH_INVENTORY)) {
					// inventory
					if (value1.equals(LESS_THAN)) {
						Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_INVENTORY_QUANTITY, value2);
						Action.Parameter subParameters[] = new Action.Parameter[1];
						subParameters[0] = new Action.Parameter(ACTION_PARAMETER_COMPARISON_OPERATOR, OPERATOR_LESS_THAN);
						parameter.setSubParameters(subParameters);
						parameterVector.addElement(parameter);
					}
				}
				else if (type.equals(LOW_PRICE)) {
					// low price
					String comparisonOperator = null;
					if (value1.equals(GREATER_THAN)) {
						comparisonOperator = OPERATOR_GREATER_THAN;
					}
					else if (value1.equals(EQUAL_TO)) {
						comparisonOperator = OPERATOR_EQUAL_TO;
					}
					else if (value1.equals(LESS_THAN)) {
						comparisonOperator = OPERATOR_LESS_THAN;
					}
					if (comparisonOperator != null) {
						Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_OFFER_PRICE, value2);
						Action.Parameter subParameters[] = new Action.Parameter[2];
						subParameters[0] = new Action.Parameter(ACTION_PARAMETER_COMPARISON_OPERATOR, comparisonOperator);
						subParameters[1] = new Action.Parameter(ACTION_PARAMETER_CURRENCY, _initiative.getCurrency());
						parameter.setSubParameters(subParameters);
						parameterVector.addElement(parameter);
					}
				}
				else if (type.equals(HIGH_PRICE)) {
					// high price
					if (value1.equals(LESS_THAN)) {
						Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_OFFER_PRICE, value2);
						Action.Parameter subParameters[] = new Action.Parameter[2];
						subParameters[0] = new Action.Parameter(ACTION_PARAMETER_COMPARISON_OPERATOR, OPERATOR_LESS_THAN);
						subParameters[1] = new Action.Parameter(ACTION_PARAMETER_CURRENCY, _initiative.getCurrency());
						parameter.setSubParameters(subParameters);
						parameterVector.addElement(parameter);
					}
				}
				else if (type.equals(AVAILABLE_AFTER)) {
					// available after
					Action.Parameter parameter = new Action.Parameter(ACTION_PARAMETER_AVAILABILITY_DATE, CampaignUtil.normalizeStringDate(value1));
					Action.Parameter subParameters[] = new Action.Parameter[1];
					subParameters[0] = new Action.Parameter(ACTION_PARAMETER_COMPARISON_OPERATOR, OPERATOR_GREATER_THAN);
					parameter.setSubParameters(subParameters);
					parameterVector.addElement(parameter);
				}
			} // end value.length
		} // end for
		parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_QUERY_OPERATOR, ACTION_PARAMETER_VALUE_AND));
		parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_QUERY_TYPE, ACTION_PARAMETER_VALUE_BOTH_QUERY));
	}
	else if (WHAT_TYPE_COLLABORATIVE_FILTERING.equals(whatType)) {
		_resulttype = INITIATIVE_RESULT_TYPE_PRODUCT;
		actionName = ACTION_COLLABORATIVE_FILTERING;
	}
	else if (WHAT_TYPE_COLLATERAL.equals(whatType)) {
		_resulttype = INITIATIVE_RESULT_TYPE_ADVERTISEMENT;
		actionName = ACTION_AWARENESS_AD;
		Vector selectedCollateral = _initiative.getSelectedCollateral();
		if (selectedCollateral.size() > 0) {
			for (int collIndex=0; collIndex<selectedCollateral.size(); collIndex++) {
				Hashtable collateralItem = (Hashtable)selectedCollateral.elementAt(collIndex);
				parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_COLLATERAL, (String) collateralItem.get("name")));
			}
		}
	}
	else if (WHAT_TYPE_DISCOUNT_COLLATERAL.equals(whatType)) {
		_resulttype = INITIATIVE_RESULT_TYPE_ADVERTISEMENT;
		actionName = ACTION_DISCOUNT_AD;
		Vector selectedDiscountCollateral = _initiative.getSelectedDiscountCollateral();
		if (selectedDiscountCollateral.size() > 0) {
			for (int collIndex=0; collIndex<selectedDiscountCollateral.size(); collIndex++) {
				Hashtable collateralDiscountItem = (Hashtable)selectedDiscountCollateral.elementAt(collIndex);
				parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_DISCOUNT_COLLATERAL, (String) collateralDiscountItem.get("name")));
			}
		}
		String selectedDiscountUsage = _initiative.getSelectedDiscountUsage();
		String selectedDiscountCode = _initiative.getSelectedDiscountCode();
		parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_DISCOUNT_USAGE, (String) selectedDiscountUsage));
		parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_DISCOUNT_CODE, (String) selectedDiscountCode));
	}
	else if (WHAT_TYPE_COUPON_COLLATERAL.equals(whatType)) {
		_resulttype = INITIATIVE_RESULT_TYPE_ADVERTISEMENT;
		actionName = ACTION_COUPON_AD;
		Vector selectedCouponCollateral = _initiative.getSelectedCouponCollateral();
		if (selectedCouponCollateral.size() > 0) {
			for (int collIndex=0; collIndex<selectedCouponCollateral.size(); collIndex++) {
				Hashtable collateralCouponItem = (Hashtable)selectedCouponCollateral.elementAt(collIndex);
				parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_COUPON_COLLATERAL, (String) collateralCouponItem.get("name")));
			}
		}
	}
	else if (WHAT_TYPE_CATEGORY.equals(whatType)) {
		_resulttype = INITIATIVE_RESULT_TYPE_CATEGORY;
		actionName = ACTION_CATEGORY_RECOMMENDATION;
		Vector selectedCategories = _initiative.getSelectedCategories();
		if (selectedCategories.size() > 0) {
			for (int i=0; i<selectedCategories.size(); i++) {
				parameterVector.addElement(new Action.Parameter(ACTION_PARAMETER_CATEGORY, (String) selectedCategories.elementAt(i)));
			}
		}
	}

	Action.Parameter[] parameters = new Action.Parameter[parameterVector.size()];
	parameterVector.copyInto(parameters);
	Action action = new Action(actionName, parameters);
	Rule rule = new Rule(condition, action);
	_initiativeRule = rule.getXMLDocument();
  }

  /**
   * This function generates the qualifier for the supplied currency.
   *
   * @return A SimpleCondition.Qualifier object of the currency.
   */
  protected SimpleCondition.Qualifier getCurrencyQualifier () {
	if (_currencyQualifier == null) {
		_currencyQualifier = new SimpleCondition.Qualifier(QUALIFIER_CURRENCY, _initiative.getCurrency());
	}
	return _currencyQualifier;
  }

  /**
   * This function generates the qualifier for the given language.
   *
   * @return A SimpleCondition.Qualifier object of the language.
   */
  protected SimpleCondition.Qualifier getLanguageQualifier () {
	if (_languageQualifier == null) {
		_languageQualifier = new SimpleCondition.Qualifier(QUALIFIER_LANGUAGE, getCommandContext().getLocale().toString());
	}
	return _languageQualifier;
  }
}